#------------------------------------------------------
#               DistatisR:
#   How to use  DISTATIS with R
#   Example from Abdi et al. 2007
#            
#------------------------------------------------------
# Hervé Abdi: herve@utdallas.edu
# http://www.utdallas.edu/~
#------------------------------------------------------
# July 23 2014
#
#-------------------------------------------------------
#-------------------------------------------------------
# If you have not installed the packages that are
# are needed, the program will install them
#
#-------------------------------------------------------
# Install the libraries that are needed
# First download the packages if needed
# This is the list of Libraries that we need
Liste2Install = c('DistatisR','XLConnect')
# Test that there are  already installed
# if not install them
ListePackages = installed.packages()
for (i in 1:length(Liste2Install)){
LePackage = Liste2Install[i]
if( !(LePackage %in% rownames(ListePackages))) {
	    # If LePackage is not installed, install it
	    print(c('Installing Package: ',Liste2Install[i]))
	    flush.console()
	    install.packages(LePackage,dependencies=TRUE,
	                          quiet=TRUE,verbose=FALSE)
	    }
	 }   # end of loop i
# activate the libraries
for (i in 1:length(Liste2Install)){# note that
	# the option "character.only = TRUE" 
	# is needed to load from a list of names
	 print(c('Install Library: ',Liste2Install[i]))
	 flush.console()
      require(Liste2Install[i],
             character.only = TRUE,quietly = TRUE)	
    }   # end of loop i
#
#-------------------------------------------------------
# The example used here comes from Table 1 of
# Abdi, H., Valentin, D., Chollet, S., & Chrea, C. (2007).
#         Analyzing assessors and products in sorting tasks: 
#         DISTATIS, theory and applications. 
#        Food Quality and Preference, 18, 627–640.
# In this Example 10 Assessors sorted 8 Beers
#----------------------------------------------------------------
#      this is the way the results look from Table 1 of 
#      Abdi et al. (2007).
#                       Assessors
#                  1 2 3 4 5 6 7 8 9 10
# Beer        Sex  f m f f m m m m f m
#            -----------------------------                         
#Affligen          1 4 3 4 1 1 2 2 1 3
#Budweiser         4 5 2 5 2 3 1 1 4 3
#Buckler_Blonde    3 1 2 3 2 4 3 1 1 2
#Killian           4 2 3 3 1 1 1 2 1 4
#St. Landelin      1 5 3 5 2 1 1 2 1 3
#Buckler_Highland  2 3 1 1 3 5 4 4 3 1
#Fruit Defendu     1 4 3 4 1 1 2 2 2 4
#EKU28             5 2 4 2 4 2 5 3 4 5
#
# In this table a column is an assessor and a row is a beer
# in a given column, 2 beers have the same number when they were sorted
# together by the assessor represented by this column.
#----------------------------------------------------------------
# 1  Read the data from a csv file   
# The data that we need are stored in a .csv file
#  whose name is stored in the variable File2Read
#----------------------------------------------------------------
File2Read = 'BeerSortingTask.csv'
# Use Read Table
Sort <- read.table(File2Read , header=TRUE, 
             sep=",", na.strings="NA", dec=".", 
                   row.names=1, strip.white=TRUE)
#-----------------------------------------------------------------------------
# 2. Create the set of distance matrices 
#     (DistatisR needs an 3 dimensional array Beers * Beers * Assessors
#      i.e., one distance matrix per assessor)
#    This set of distance matrices 
#       is created by the function: DistanceFromSort
DistanceCube <- DistanceFromSort(Sort)
#-----------------------------------------------------------------------------
# 3.0 Before DistatisR, we want to run a plain metric MDS
#     3.0.1. Create the distance matrix
#            The distance matrix is obtained by add
#            All the assessors individual distance matrices
#            This done by this instruction that create
#            the distance matrix called TotalDistance
TotalDistance = apply(DistanceCube,c(1,2),sum)
#     3.0.2. Analyze the distance matrix TotalDistance by mds
#            To do so we use the function mmds from DistatisR
mdsRes <- mmds(TotalDistance)
#            the results are stored in the list mdsRes
#     3.0.3. We plot the results of the MDS using the func prettyPlot
#            from the package prettyGraphs.
#            For this graph, we use factor scores stored in mdsRes$FactorScore
plotMDS <- prettyPlot(mdsRes$FactorScore,
                      display_names = TRUE,
                      display_points = TRUE,
                      contributionCircles = FALSE,
                      contributions = mdsRes$Contributions
                      )
#-----------------------------------------------------------------------------
# Now Go for Distance proper
# 3. Call the DISTATIS routine with the cube of distance as a parameter
testDistatis <- distatis(DistanceCube)
# The factor scores for the beers are in
# testDistatis$res4Splus$F
# the factor scores for the assessors are in (RV matrice)
#  testDistatis$res4Cmat$G

#-----------------------------------------------------------------------------
# 4. Inferences on the beers obtained via bootstrap
#    here we use two different bootstraps:
#    1. Bootstrap on factors (very fast but could be too liberal 
#         when the number of assessors is very large)
#    2. Complete bootstrap obtained by computing sets of compromises
#       and projecting them (could be significantly longer because a lot
#       of computations is required)
# 
# 4.1 Get the bootstrap factor scores (with default 1000 iterations)
BootF <- BootFactorScores(testDistatis$res4Splus$PartialF)
#
# 4.2 Get the boostrap from full bootstrap (default niter = 1000)
 F_fullBoot <- BootFromCompromise(DistanceCube,niter=1000) 
 

#-----------------------------------------------------------------------------
# 5. Create the Graphics
# Here we produce only the most important three graphs
#      to get the rest of the graphs can be produced by de-commenting the
#       5.3.,  5.4.2., and 5.5
# 5.1 an Rv map
 rv.graph.out <- GraphDistatisRv(testDistatis$res4Cmat$G) 
# 5.2 a compromise plot
 compromise.graph.out <- GraphDistatisCompromise(testDistatis$res4Splus$F) 
# 5.3 a partial factor score plot
# partial.scores.graph.out <- 
# 	GraphDistatisPartial(testDistatis$res4Splus$F,testDistatis$res4Splus$PartialF)
# 5.4 a bootstrap confidence interval plot 
 #5.4.1 with ellipses
 boot.graph.out.ell <- GraphDistatisBoot(testDistatis$res4Splus$F,BootF)  
 #or 
 # boot.graph.out <- GraphDistatisBoot(testDistatis$res4Splus$F,F_fullBoot)  
 #5.4.2 with hulls
 # boot.graph.out.hull <- GraphDistatisBoot(testDistatis$res4Splus$F,BootF,ellipses=FALSE)  
 #or 
 # boot.graph.out <- GraphDistatisBoot(testDistatis$res4Splus$F,F_fullBoot,ellipses=FALSE)
#5.5 all the plots at once
#all.plots.out <- 
#	GraphDistatisAll(testDistatis$res4Splus$F,testDistatis$res4Splus$PartialF,
#		BootF,testDistatis$res4Cmat$G)  
#-----------------------------------------------------------------------------

